/*
	Copyright (c) 2004 Pablo Bleyer Kocik.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	1. Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.

	2. Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.

	3. The name of the author may not be used to endorse or promote products
	derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS OR IMPLIED
	WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
	MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
	EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
	BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
	IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
	POSSIBILITY OF SUCH DAMAGE.
*/

/*
	kcpsmx test
*/

// synthesis attribute LOC   MCKO "P55"
// synthesis attribute LOC   CLK0 "P129"
// synthesis attribute LOC   CLK1 "P126"
// synthesis attribute LOC   CLKX "P52"
// synthesis attribute LOC   CSDA "P140"
// synthesis attribute LOC   CSCL "P139"
// synthesis attribute LOC   CTL0 "P142"
// synthesis attribute LOC   CTL1 "P141"
// synthesis attribute LOC   nCS0 "P79"
// synthesis attribute LOC   nCS1 "P78"
// synthesis attribute LOC   nCS2 "P77"
// synthesis attribute LOC   nCS3 "P76"
// synthesis attribute LOC    nUB "P83"
// synthesis attribute LOC    nWE "P84"
// synthesis attribute LOC    nOE "P85"
// synthesis attribute LOC  nWAIT "P87"
// synthesis attribute LOC      A "P30 P31 P32 P38 P39 P40 P41 P42 P43 P44 P47 P48 P49 P57 P58 P59 P60 P63 P64 P65 P66 P67 P68 P69"
// synthesis attribute LOC      D "P10 P11 P12 P13 P14 P15 P18 P20 P21 P22 P23 P24 P26 P27 P28 P29"
// synthesis attribute LOC    IOA "P95 P96 P97 P101 P102 P103 P114 P115 P116 P117 P118 P121 P122 P123"
// synthesis attribute LOC    IOB "P104 P74 P105 P100 P98 P94 P86 P82 P80 P75 P113 P112 P106 P124 P56 P50 P131 P125"
// synthesis attribute LOC   IRQ0 "P138"
// synthesis attribute LOC   IRQ1 "P137"
// synthesis attribute LOC   IRQ2 "P134"
// synthesis attribute LOC   IRQ3 "P133"
// synthesis attribute LOC    FIQ "P132"
// synthesis attribute LOC   SCK0 "P3"
// synthesis attribute LOC   TXD0 "P4"
// synthesis attribute LOC   RXD0 "P5"
// synthesis attribute LOC   SCK1 "P6"
// synthesis attribute LOC   TXD1 "P7"
// synthesis attribute LOC   RXD1 "P8"
// synthesis attribute LOC   nRST "P92"
// synthesis attribute LOC nWDOVF "P93"
// synthesis attribute LOC    PME "P89"

`include "../kcpsmx/timescale_inc.v"
`include "../kcpsmx/kcpsmx_inc.v"

module kcpsmx_uclock(
	MCKO,
	nRST,
	A, D,
	IOA
);

input MCKO, nRST;
input [23:0] A;
input [15:0] D;
inout [13:0] IOA;

wire clk, rst, ir; // clock, reset, interrupt req
wire [`code_depth-1:0] ad; // instruction address
wire [`operand_width-1:0] pa, po; // port id, port out
wire rd, wr, ia; // read strobe, write strobe, interrupt ack

wire [`code_width-1:0] di;
wire [`operand_width-1:0] pi;

assign clk = MCKO;
assign rst = !nRST;
assign pi = D[7:0];
assign IOA[7:0] = po;
assign IOA[8] = 'hz;
assign ir = IOA[8];


uclock rom(
	.clk(clk),
	.address(ad),
	.instruction(di)
);

kcpsmx dut(
	.clk(clk),
	.reset(rst),
	.address(ad),
	.instruction(di),
	.port_id(pa),
	.read_strobe(rd),
	.write_strobe(wr),
	.in_port(pi),
	.out_port(po),
	.interrupt(ir),
	.interrupt_ack(ia)
);

endmodule
